#include "StdAfx.h"
#include "FTState.h"
#include "FTSLayoutOffset.h"
#include "ChatItem.h"
#include "FTImageMap.h"
#include "UIChatFileElement.h"
#include "../DuiLib/Control/UIRichEdit.h"
#include "ChatDlg.h"
#include "ChatMainView.h"


using namespace  ftsLayOutOffset;

namespace FileTransferState{

	CFTState::CFTState(UIChatFileElement * pUIElem)
		:m_pUIChatFileElement(pUIElem)
	{
		ASSERT(m_pUIChatFileElement);
		ASSERT(m_pUIChatFileElement->GetElementResource());

        m_BubbleInnerWidth = 0.0f;

		int nDisplayType = pUIElem->GetElementResource()->GetChatDisplayType();
		m_alignmentFileName = Gdiplus::StringAlignmentNear;
		m_clrFileName = GetCurrentLayout(nDisplayType).common().clr_text_normal;
		m_fontStyleFileName = FontStyleRegular;
		m_bWillDisplayFileSize = FALSE;
		m_bFileNameIsLink = FALSE;

		m_strProgressInfo.Empty();
		m_strProgressExtraStatus.Empty();

		m_bShowDivider = FALSE;
		m_alignmentProgress = Gdiplus::StringAlignmentNear;

		for (INT btnIndex = (INT)Button_General; btnIndex < (INT)Button_Count; btnIndex++)
		{
			//m_rcButton[btnIndex]
			m_strButtonText[btnIndex].Empty();
			m_alignmentButton[btnIndex] = Gdiplus::StringAlignmentNear;
			m_clrButtonText[btnIndex] = GetCurrentLayout(nDisplayType).common().clr_text_normal;
			m_fontStyleButtonText[btnIndex] = FontStyleRegular;
			m_bButtonIsLink[btnIndex] = FALSE;
			m_bCheckDoubleClick[btnIndex] = FALSE;
			m_bButtonClicked[btnIndex] = FALSE;
		}
	}

	CFTState::~CFTState()
	{
		m_pUIChatFileElement = NULL;
	}

	void CFTState::_InitControlValue()
	{
	}

	void CFTState::SetFocusToChatEdit()
	{
		CRichEditUI *pRichEdit =  m_pUIChatFileElement->GetChatMainView()->GetChatDlg()->GetChatEdit();
		if (pRichEdit)
		{
			pRichEdit->SetFocus();
		}
	}

	void CFTState::OnLButtonDown(UINT nFlags, CPoint point)
	{
		if (m_bFileNameIsLink && m_rcFileName.Contains((REAL)point.x, (REAL)point.y))
		{
			_OnFileNameClick();
		}
		if (m_bButtonIsLink[Button_General] && m_rcButton[Button_General].Contains((REAL)point.x, (REAL)point.y))
		{
			if (!m_bCheckDoubleClick[Button_General] || !m_bButtonClicked[Button_General])
			{
				//Double Click Check -- Retry or StartDownload
				m_bButtonClicked[Button_General] = TRUE;
				_OnButtonClick();
			}
		}
	}
	void CFTState::OnLButtonUp(UINT nFlags, CPoint point)
	{

	}

	BOOL CFTState::OnMouseMove(UINT nFlags, CPoint point)
	{
		BOOL bHandled = FALSE; 
		//TODO: m_rcItem
		if (m_bFileNameIsLink && m_rcFileName.Contains((REAL)point.x, (REAL)point.y))
		{
			bHandled = TRUE;
			m_pUIChatFileElement->GetElementResource()->SetCursor(CURSOR_HAND);
		}
		else if (m_bButtonIsLink[Button_General] && m_rcButton[Button_General].Contains((REAL)point.x, (REAL)point.y))
		{
			bHandled = TRUE;
			m_pUIChatFileElement->GetElementResource()->SetCursor(CURSOR_HAND);
		}

		else
		{
			m_pUIChatFileElement->GetElementResource()->SetCursor(CURSOR_DEFAULT);
		}
		return bHandled;
	}

	CSize CFTState::CalcSize(const CPoint& ptPos, DWORD dwFlags,  CRect& rc)
	{
		CSize sz( 0, 0);
		_InitControlValue();
		CPoint ptOffset = ptPos;

		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		int nType = pResource->GetChatDisplayType();
		if(CHAT_BUBBLE == nType)
		{
			_SetControlPos(ptOffset, sz, dwFlags);
		}
		
		_CalcFileNameSize(ptOffset);
		_CalcFileIconSize(ptOffset);
		
		_CalcProgressSize(ptOffset);
		_CalcProgressExtraStatusSize(ptOffset);
		_CalcMyDividerSize(ptOffset);
		_CalcMyButtonsSize(ptOffset);

		if(CHAT_PLAIN == nType)
		{
			_SetControlPos(ptOffset, sz, dwFlags);
		}
		sz.cx = m_rcFileIcon.Width;
		sz.cy = m_rcFileIcon.Height;

		rc.SetRect(m_rcFileIcon.X, m_rcFileIcon.Y, m_rcFileIcon.GetRight(), m_rcFileIcon.GetBottom());
		pResource->SetTestChatDisplayType(nType);
		return sz;
	}

	CSize CFTState::_CalcTextSize(const CString& strInfo, const CPoint& ptOffset, REAL fontHeight)
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		CSize szBound( 0, 0);
		
		RecalcTextRect(pResource->GetCurrentWnd(), strInfo, g_defaultFontFaceName, fontHeight, 
			FontStyleRegular, pResource->GetViewRect(), szBound);
		return szBound;
	}

	//CString CFTState::_CalcDisplayFileNameString()
	//{
	//	CChatFileItem* pChatFileItem = m_pUIChatFileElement->GetChatFileItem();
	//	const AttachFileInfo& fileInfo = pChatFileItem->GetTranslateFileInfo()->m_fileInfo;

	//	CString strDisplayFileName = fileInfo.fileName.c_str();
	//	INT nFileSize = (INT)fileInfo.fileSize; //small than 2G
	//	if (m_bWillDisplayFileSize)
	//	{
	//		if (nFileSize >= 1024)
	//		{
	//			strDisplayFileName.Format(TEXT("%s(%dkb)"), fileInfo.fileName.c_str(), (nFileSize / 1024));
	//		}
	//		else
	//		{
	//			strDisplayFileName.Format(TEXT("%s(%dByte)"), fileInfo.fileName.c_str(), nFileSize);
	//		}
	//	}
	//	
	//	IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();

	//	RectF rcDraw(0, 0, GetCurrentLayout().common().filename_max_width, GetCurrentLayout().common().font_height);
	//	CRect rcText(rcDraw.X, rcDraw.Y, rcDraw.GetRight(), rcDraw.GetBottom());

	//	strDisplayFileName = RecalcTextString(pResource->GetCurrentWnd(), strDisplayFileName, DEFAULT_KOREAN_FONT_NAME, 
	//		GetCurrentLayout().common().font_height, m_fontStyleFileName, rcText);

	//	return strDisplayFileName;
	//}

	CString CFTState::_CalcDisplayFileNameString( RectF rcDraw )
	{
		CChatFileItem* pChatFileItem = m_pUIChatFileElement->GetChatFileItem();
		const AttachFileInfo& fileInfo = pChatFileItem->GetTranslateFileInfo()->m_fileInfo;

		CString strDisplayFileName = fileInfo.fileName.c_str();
		INT nFileSize = (INT)fileInfo.fileSize; //small than 2G
		if (m_bWillDisplayFileSize)
		{
			if (nFileSize >= 1024 * 1024)
			{
				int nRemainder = nFileSize % (1024 * 1024);
				if (nRemainder != 0)
				{
					float fFileSize = (float)nFileSize / (1024.0f * 1024.0f);
					strDisplayFileName.Format(TEXT("%s (%0.1fmb)"), fileInfo.fileName.c_str(), fFileSize);
				}
				else
				{
					strDisplayFileName.Format(TEXT("%s (%dmb)"), fileInfo.fileName.c_str(), nFileSize / (1024 * 1024));
				}

			}
			else if (nFileSize >= 1024)
			{
				int nRemainder = nFileSize % 1024;
				if (nRemainder != 0)
				{
					float fFileSize = (float)nFileSize / 1024.0f;
					strDisplayFileName.Format(TEXT("%s (%0.1fkb)"), fileInfo.fileName.c_str(), fFileSize);
				}
				else
				{
					strDisplayFileName.Format(TEXT("%s (%dkb)"), fileInfo.fileName.c_str(), nFileSize / 1024);
				}
			}
			else
			{
				strDisplayFileName.Format(TEXT("%s (%dByte)"), fileInfo.fileName.c_str(), nFileSize);
			}
		}

		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		if(CHAT_PLAIN == pResource->GetChatDisplayType())
		{
			CRect rcText(rcDraw.X, rcDraw.Y, rcDraw.GetRight(), rcDraw.GetBottom());
			REAL fontHeight = 8;
			strDisplayFileName = RecalcTextString(pResource->GetCurrentWnd(), strDisplayFileName, g_defaultFontFaceName, 
				fontHeight, m_fontStyleFileName, rcText);
		}
		

		return strDisplayFileName;
	}

	void CFTState::_CalcFileIconSize(const CPoint& ptOffset)
	{
		int nChatType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
		const commonInfo& cmmInfo = GetCurrentLayout(nChatType).common();
		
		if(nChatType == CHAT_BUBBLE)
		{
			m_rcFileIcon.X = ptOffset.x;
			m_rcFileIcon.Y = ptOffset.y;

			m_BubbleInnerWidth = m_rcFileName.Width;
			if(m_BubbleInnerWidth < m_nBubble_Inner_width_min)
			{
				m_BubbleInnerWidth = m_nBubble_Inner_width_min;
				m_rcFileName.X += (m_BubbleInnerWidth - (int)m_rcFileName.Width) / 2;
			}
			m_rcFileIcon.Width = m_BubbleInnerWidth + _GetBubbleImage(FALSE, 0)->GetWidth() + _GetBubbleImage(FALSE, 2)->GetWidth();
				/*((int)m_rcFileName.Width + (int)_GetBubbleImage(FALSE, 0)->GetWidth()
				+ (int)_GetBubbleImage(FALSE, 2)->GetWidth() - (int)m_rcFileIcon.Width) / 2;*/
			m_rcFileIcon.Height = cmmInfo.icon_file_type_height + cmmInfo.GetFileIconEdgeY() + 
				cmmInfo.filename_to_fileicon_top + m_rcFileName.Height + 
				cmmInfo.GetFileNameInnerEdgeY() + _GetBubbleImage(FALSE, 6)->GetHeight();
		}
		else
		{
			m_rcFileIcon.Width = cmmInfo.GetFileIconWidth();//(REAL)(cmmInfo.bubble_to_image_left + cmmInfo.icon_file_type_width + cmmInfo.bubble_to_image_right);
			m_rcFileIcon.Height = cmmInfo.GetFileIconHeight();//(REAL)(cmmInfo.bubble_to_image_top + cmmInfo.icon_file_type_height + cmmInfo.bubble_to_image_bottom);
		}
	}

	void CFTState::_CalcFileNameSize(const CPoint& ptOffset)
	{
		//const commonInfo& cmmInfo = GetCurrentLayout().common();
		//CString strDisplayFileName = _CalcDisplayFileNameString(m_rcFileName);
		//CSize szText = _CalcTextSize(strDisplayFileName, ptOffset);
		//m_rcFileName.Width = (REAL)min(cmmInfo.filename_max_width, szText.cx);
		//m_rcFileName.Height = (REAL)szText.cy;// GetCurrentLayout().common().filename_fontheight_offset;
	}

	void CFTState::_CalcMyProgressSize(const CPoint& ptOffset)
	{

		
		
	}

	void CFTState::_CalcProgressExtraStatusSize(const CPoint& ptOffset)
	{
		if (!m_strProgressExtraStatus.IsEmpty())
		{
			IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
			int nDisplayType = pResource->GetChatDisplayType();
			REAL nFontHeight = GetCurrentLayout(nDisplayType).common().font_height;
			CSize szBound( 0, 0);


			RecalcTextRect(pResource->GetCurrentWnd(), m_strProgressExtraStatus, g_defaultFontFaceName, nFontHeight, 
				FontStyleRegular, pResource->GetViewRect(), szBound);

			m_rcProgressExtraStatus.Width = (REAL)szBound.cx;
			m_rcProgressExtraStatus.Height = (REAL)szBound.cy;
		}
		else
		{
			m_rcProgressExtraStatus.Width = 0.0f;
			m_rcProgressExtraStatus.Height = 0.0f;
		}
	}
	void CFTState::_CalcMyDividerSize(const CPoint& ptOffset)
	{
		int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
		if(nDisplayType == CHAT_BUBBLE)
		{
			return _CalcDividerSize(ptOffset);
		}
		m_rcDivider.Width = (REAL)GetCurrentLayout(nDisplayType).common().divider_width;
		m_rcDivider.Height = (REAL)GetCurrentLayout(nDisplayType).common().divider_height;
	}

	void CFTState::_CalcMyButtonsSize(const CPoint& ptOffset)
	{
		int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
		if(nDisplayType == CHAT_BUBBLE)
		{
			return _CalcButtonsSize(ptOffset);
		}
		if (!m_strButtonText[Button_General].IsEmpty())
		{
			IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();

			REAL nFontHeight = GetCurrentLayout(nDisplayType).common().font_height;
			CSize szBound( 0, 0);

			RecalcTextRect(pResource->GetCurrentWnd(), m_strButtonText[Button_General], g_defaultFontFaceName, nFontHeight, 
				FontStyleRegular, pResource->GetViewRect(), szBound);
			m_rcButton[Button_General].Width = (REAL)szBound.cx;
			m_rcButton[Button_General].Height = (REAL)szBound.cy;
		}
		else
		{
			m_rcButton[Button_General].Width = (REAL)GetCurrentLayout(nDisplayType).common().icon_cancel_height;
			m_rcButton[Button_General].Height = (REAL)GetCurrentLayout(nDisplayType).common().icon_cancel_height;
		}
	}

	BOOL CFTState::DrawItems(Graphics *pGraph)
	{
		CPoint ptOffset = m_pUIChatFileElement->GetElementResource()->GetOffSetPoint();
		PointF ptGdiPOffset((REAL)-ptOffset.x, (REAL)-ptOffset.y); 
		
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		int nOldType = pResource->GetChatDisplayType();
		//pResource->SetTestChatDisplayType(CHAT_BUBBLE);

		_DrawFileIcon(pGraph, ptGdiPOffset);
		_DrawFileName(pGraph, ptGdiPOffset);
		
		_DrawProgress(pGraph, ptGdiPOffset);
		_DrawProgressExtraStatus(pGraph, ptGdiPOffset);
		_DrawDivider(pGraph, ptGdiPOffset);
		_DrawMyButton(pGraph, ptGdiPOffset);

		_DrawRectBounds(pGraph, ptGdiPOffset);

		pResource->SetTestChatDisplayType(nOldType);
		return TRUE;
	}

	BOOL CFTState::_DrawRectBounds(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
#if 0
		Gdiplus::Pen pen(ARGB(Color::Red), 1.0f);

		RectF rcDraw = m_rcFileName;
		rcDraw.Offset(ptGdiPOffset);
		pGraph->DrawRectangle(&pen, rcDraw);

		rcDraw = m_rcFileIcon;
		rcDraw.Offset(ptGdiPOffset);
		pGraph->DrawRectangle(&pen, rcDraw);

		rcDraw = m_rcProgress;
		rcDraw.Offset(ptGdiPOffset);
		pGraph->DrawRectangle(&pen, rcDraw);
		
		rcDraw = m_rcProgressExtraStatus;
		rcDraw.Offset(ptGdiPOffset);
		pGraph->DrawRectangle(&pen, rcDraw);
#endif
		return TRUE;
	}

	BOOL CFTState::_DrawFileName(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		RectF rcDraw = m_rcFileName;
		rcDraw.Offset(ptGdiPOffset);

		CString strDisplayFileName = _CalcDisplayFileNameString(m_rcFileName);
		
		//Gdiplus::Font fontFileName(DEFAULT_KOREAN_FONT_NAME, GetCurrentLayout().common().font_height);
		//StringFormat format;
		//format.SetAlignment(m_alignmentFileName);
		//SolidBrush brush(m_clrFileName);
		//pGraph->DrawString(strDisplayFileName, strDisplayFileName.GetLength(), &fontFileName, rcDraw, &format, &brush);

		CRect rcText(rcDraw.X, rcDraw.Y, rcDraw.GetRight(), rcDraw.GetBottom());
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		CRect rcClient = pResource->GetViewRect();

		REAL fontHeight = GetCurrentLayout(pResource->GetChatDisplayType()).common().font_height;
		DrawTextToDest2(pGraph, strDisplayFileName, &rcClient, &rcText, g_defaultFontFaceName,
			fontHeight, m_alignmentFileName, m_fontStyleFileName, m_clrFileName);

		//UIFileNameDescription m_filename;
		//m_filename.DrawFileName(m_pUIChatFileElement, rcDraw, m_bWillDisplayFileSize);

		return TRUE;
	}
	BOOL CFTState::_DrawProgress(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		if (!m_rcProgress.IsEmptyArea())
		{
			RectF rcDraw = m_rcProgress;
			rcDraw.Offset(ptGdiPOffset);
			IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
			int nDisplayType = pResource->GetChatDisplayType();
			if (m_strProgressInfo.IsEmpty())
			{
				if (CHAT_BUBBLE  == nDisplayType)
				{
					//rcDraw.Y += GetCurrentLayout(nDisplayType).common().progressbar_top;
					_DrawProgressBar(pGraph, rcDraw);
				}
				else if (CHAT_PLAIN == nDisplayType)
				{
					_DrawProgressText(pGraph, rcDraw);
				}
			}
			else
			{
				//draw text
				const commonInfo& comm = GetCurrentLayout(nDisplayType).common();
				CRect rcText(rcDraw.X, rcDraw.Y, rcDraw.GetRight(), rcDraw.GetBottom());
				rcText.OffsetRect(0,2);
				CRect rcClient = pResource->GetViewRect();
				REAL fontHeight = comm.font_height;
				//SolidBrush txtbrush();
				DrawTextToDest2(pGraph, m_strProgressInfo, &rcClient, &rcText, g_defaultFontFaceName,
					fontHeight, m_alignmentProgress, FontStyleRegular, CColorConvert::HEX2Color(0x000000));
			}
		}
		return TRUE;
	}

	BOOL CFTState::_DrawProgressExtraStatus(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		if (!m_strProgressExtraStatus.IsEmpty() && !m_rcProgressExtraStatus.IsEmptyArea())
		{
			int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
			RectF rcDraw = m_rcProgressExtraStatus;
			rcDraw.Offset(ptGdiPOffset);

			StringFormat format;
			format.SetAlignment(StringAlignmentCenter);
			Gdiplus::Font txtfont(g_defaultFontFaceName, GetCurrentLayout(nDisplayType).common().font_height, FontStyleRegular );
			SolidBrush txtbrush(GetCurrentLayout(nDisplayType).common().clr_text_normal);
			pGraph->DrawString(m_strProgressExtraStatus, m_strProgressExtraStatus.GetLength(), 
				&txtfont, rcDraw, &format,  &txtbrush);
		}
		return TRUE;
	}

	BOOL CFTState::_DrawDivider(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		if (m_bShowDivider && !m_rcDivider.IsEmptyArea())
		{
			int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
			if (CHAT_BUBBLE == nDisplayType)
			{
				InterpolationMode interpolationMode = pGraph->GetInterpolationMode();
				PixelOffsetMode pixelOffsetMode = pGraph->GetPixelOffsetMode();
				pGraph->SetInterpolationMode(InterpolationModeBilinear);
				pGraph->SetPixelOffsetMode(PixelOffsetModeNone);

				RectF rcDraw = m_rcDivider;
				rcDraw.Offset(ptGdiPOffset);

				pGraph->DrawImage(m_pUIChatFileElement->GetElementResource()->GetDividerImage(),rcDraw);
				
				pGraph->SetInterpolationMode(interpolationMode);
				pGraph->SetPixelOffsetMode(pixelOffsetMode);
			}
			else
			{
				InterpolationMode interpolationMode = pGraph->GetInterpolationMode();
				PixelOffsetMode pixelOffsetMode = pGraph->GetPixelOffsetMode();
				pGraph->SetInterpolationMode(InterpolationModeNearestNeighbor);
				pGraph->SetPixelOffsetMode(PixelOffsetModeHalf);

				int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
				SolidBrush brush(GetCurrentLayout(nDisplayType).common().clr_divider);
				RectF rcDraw = m_rcDivider;
				rcDraw.Offset(ptGdiPOffset);

				pGraph->FillRectangle(&brush, rcDraw);

				pGraph->SetInterpolationMode(interpolationMode);
				pGraph->SetPixelOffsetMode(pixelOffsetMode);
			}
			
		}
		return TRUE;
	}

	BOOL CFTState::_DrawFileIcon(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
		if (CHAT_BUBBLE == nDisplayType)
		{
			_DrawBubbleFileIcon(pGraph, ptGdiPOffset);
		}
		else if (CHAT_PLAIN == nDisplayType)
		{
			RectF rcDraw = m_rcFileIcon;
			rcDraw.Offset(ptGdiPOffset);

			IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
			TranslateFileInfoPtr pInfo = m_pUIChatFileElement->GetChatFileItem()->GetTranslateFileInfo();
			wstring fileName = pInfo->m_fileInfo.fileName;
			Image* fIcon = pResource->GetFileTransferFileImage((LPWSTR)fileName.c_str());
			pGraph->DrawImage(fIcon, rcDraw);
		}
		return TRUE;
	}

	BOOL CFTState::_DrawMyButton(Graphics *pGraph, const PointF& ptGdiPOffset)
	{
		int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
		if(nDisplayType == CHAT_BUBBLE)
		{
			return _DrawButton(pGraph, ptGdiPOffset);
		}
		RectF rcDraw = m_rcButton[Button_General];
		rcDraw.Offset(ptGdiPOffset);
		if (m_strButtonText[Button_General].IsEmpty())
		{
			_DrawCancelButton(pGraph, ptGdiPOffset);
		}
		else
		{
			////draw text
			CRect rcText(rcDraw.X, rcDraw.Y, rcDraw.GetRight(), rcDraw.GetBottom());
			IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
			CRect rcClient = pResource->GetViewRect();
			DrawTextToDest2(pGraph, m_strButtonText[Button_General], &rcClient, &rcText, g_defaultFontFaceName,
				GetCurrentLayout(nDisplayType).common().font_height, 
				m_alignmentButton[Button_General], m_fontStyleButtonText[Button_General], m_clrButtonText[Button_General]);

			//Gdiplus::Font fontFileName(DEFAULT_KOREAN_FONT_NAME, GetCurrentLayout().common().font_height);
			//StringFormat format;
			//format.SetAlignment(m_alignmentFileName);
			//SolidBrush brush(m_clrFileName);
			//pGraph->DrawString(strDisplayFileName, strDisplayFileName.GetLength(), &fontFileName, rcDraw, &format, &brush);

		}

		return TRUE;
	}

	//BOOL CFTState::_DrawProgressBar( Graphics *pGraph, RectF& rcBar )
	//{
	//	IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
	//	Image *progress_left_bar_gray = pResource->GetFileTransferImageByID(CFTImageMap::progress_left_bar_gray);
	//	Image *progress_left_bar_highlight = pResource->GetFileTransferImageByID(CFTImageMap::progress_left_bar_highlight);
	//	Image *progress_right_bar_gray = pResource->GetFileTransferImageByID(CFTImageMap::progress_right_bar_gray);
	//	Image *progress_right_bar_highlight = pResource->GetFileTransferImageByID(CFTImageMap::progress_right_bar_highlight);
	//	Image *progress_step_bar_gray = pResource->GetFileTransferImageByID(CFTImageMap::progress_step_bar_gray);
	//	Image *progress_step_bar_highlight = pResource->GetFileTransferImageByID(CFTImageMap::progress_step_bar_highlight);
	//
	//	ASSERT(progress_left_bar_gray);
	//	ASSERT(progress_left_bar_highlight);
	//	ASSERT(progress_right_bar_gray);
	//	ASSERT(progress_right_bar_highlight);
	//	ASSERT(progress_step_bar_gray);
	//	ASSERT(progress_step_bar_highlight);

	//	if (progress_left_bar_gray == NULL
	//		|| progress_left_bar_highlight == NULL
	//		|| progress_right_bar_gray == NULL
	//		|| progress_right_bar_highlight == NULL
	//		|| progress_step_bar_gray == NULL
	//		|| progress_step_bar_highlight == NULL)
	//	{
	//		return FALSE;
	//	}

	//	int nPercent = m_pUIChatFileElement->GetProgress();

	//	pGraph->SetInterpolationMode(InterpolationModeNearestNeighbor);
	//	pGraph->SetPixelOffsetMode(PixelOffsetModeHalf);
	//	CRect rcProgressBar;
	//	rcProgressBar.bottom = rcBar.Y + rcBar.Height;
	//	rcProgressBar.top = rcProgressBar.bottom - progress_left_bar_gray->GetHeight();
	//	rcProgressBar.left = rcBar.X;
	//	rcProgressBar.right = rcBar.X + rcBar.Width;

	//	CRect rcLeft = rcProgressBar;
	//	rcLeft.right = rcLeft.left + progress_left_bar_gray->GetWidth();

	//	CRect rcRight = rcProgressBar;
	//	rcRight.left  = rcRight.right - progress_right_bar_gray->GetWidth();

	//	CRect rcStep(rcLeft.right, rcLeft.top, rcRight.left, rcRight.bottom);

	//	pGraph->DrawImage(progress_left_bar_gray, rcLeft.left, rcLeft.top, rcLeft.Width(), rcLeft.Height());
	//	pGraph->DrawImage(progress_step_bar_gray, rcStep.left, rcStep.top, rcStep.Width(), rcStep.Height());
	//	pGraph->DrawImage(progress_right_bar_gray, rcRight.left, rcRight.top, rcRight.Width(), rcRight.Height());

	//	if (nPercent > 0)
	//	{
	//		int nWidth = rcProgressBar.Width();
	//		int nStepLength = min(nWidth * nPercent / 100, nWidth);
	//		if (nStepLength > 0)
	//		{
	//			pGraph->DrawImage(progress_left_bar_highlight, 
	//				rcLeft.left, rcLeft.top, rcLeft.Width(), rcLeft.Height());
	//			nStepLength -= rcLeft.Width();
	//			if (nStepLength > 0)
	//			{
	//				pGraph->DrawImage(progress_step_bar_highlight, 
	//					rcStep.left, rcStep.top, nStepLength - rcLeft.Width(), rcStep.Height());
	//				if (nPercent == 100)
	//				{
	//					pGraph->DrawImage(progress_right_bar_highlight, 
	//						rcRight.left, rcRight.top, rcRight.Width(), rcRight.Height());
	//				}
	//			}
	//		}

	//	}
	//	return TRUE;
	//}


	BOOL CFTState::_DrawProgressBar( Graphics *pGraph, RectF& rcBar )
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();

		Image* pProgress0 = pResource->GetYourProgressBarImage(0);
		Image* pProgress1 = pResource->GetYourProgressBarImage(1);
		Image* pProgress2 = pResource->GetYourProgressBarImage(2);
		Image* pProgress3 = pResource->GetYourProgressBarImage(3);

		if (pProgress0 == NULL || pProgress1 == NULL
			|| pProgress2 == NULL || pProgress3 == NULL)
		{
			return FALSE;
		}

		int nPercent = m_pUIChatFileElement->GetProgress();

		InterpolationMode interpolationMode = pGraph->GetInterpolationMode();
		PixelOffsetMode pixelOffsetMode = pGraph->GetPixelOffsetMode();
		pGraph->SetInterpolationMode(InterpolationModeNearestNeighbor);
		pGraph->SetPixelOffsetMode(PixelOffsetModeHalf);
        CRect rcProgressBar(0, 0, 0, 0);
		rcProgressBar.bottom = rcBar.Y + rcBar.Height;
		rcProgressBar.top = rcProgressBar.bottom - pProgress1->GetHeight();
		rcProgressBar.left = rcBar.X;
		rcProgressBar.right = rcBar.X + rcBar.Width;

		//draw bg  (1,2,3)
		pGraph->DrawImage(pProgress1, rcProgressBar.left, rcProgressBar.top, pProgress1->GetWidth(), pProgress1->GetHeight());
		pGraph->DrawImage(pProgress2, rcProgressBar.left + pProgress1->GetWidth(), rcProgressBar.top, 
			rcProgressBar.Width() - pProgress1->GetWidth() - pProgress3->GetWidth(), pProgress2->GetHeight());
		pGraph->DrawImage(pProgress3, rcProgressBar.right - pProgress3->GetWidth(), rcProgressBar.top, pProgress3->GetWidth(), pProgress3->GetHeight());

		if (nPercent > 0)
		{
			int nWidth = rcProgressBar.Width();
			int nStepLength = min(nWidth * nPercent / 100, nWidth);
			if (nStepLength > 0)
			{
				pGraph->DrawImage(pProgress0, 
					rcProgressBar.left, rcProgressBar.top, nStepLength, rcProgressBar.Height());
			}
		}
		pGraph->SetInterpolationMode(interpolationMode);
		pGraph->SetPixelOffsetMode(pixelOffsetMode);
		return TRUE;
	}

	BOOL CFTState::_DrawBubbleFileIcon( Graphics *pGraph, const PointF& ptGdiPOffset )
	{
		DWORD dwFlags = m_pUIChatFileElement->GetFlags();
		RectF rcDraw = m_rcFileIcon;
		rcDraw.Offset(ptGdiPOffset);
		_DrawBubble(pGraph, rcDraw, dwFlags & DEF_CHAT_MSG_ISMINE);
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();

		TranslateFileInfoPtr pInfo = m_pUIChatFileElement->GetChatFileItem()->GetTranslateFileInfo();
		wstring fileName = pInfo->m_fileInfo.fileName;
		Image* fIcon = pResource->GetFileTransferFileImage(fileName.c_str());
		
		const commonInfo& cmmInfo = GetCurrentLayout(pResource->GetChatDisplayType()).common();

		RectF rcFileIcon;
		rcFileIcon.X = rcDraw.X + (rcDraw.Width - cmmInfo.icon_file_type_width) / 2;
		rcFileIcon.Y = rcDraw.Y + cmmInfo.GetFileIconEdgeY();
		rcFileIcon.Width = cmmInfo.icon_file_type_width;
		rcFileIcon.Height = cmmInfo.icon_file_type_height;
		pGraph->DrawImage(fIcon, rcFileIcon);
		return TRUE;
	}

	void CFTState::_DrawBubble( Graphics* graph, RectF rcBubble, BOOL bMine )
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		if (pResource)
		{
			Image *pImage[48] = {NULL};
			for (int nIndex = 0; nIndex < 9; nIndex++)
			{
				IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();

				if(bMine && nIndex > 5 && nIndex < 9)
				{
					pImage[nIndex] = pResource->GetBubbleBottomForSendProcessImage(nIndex - 6);
				}
				else
				{
					pImage[nIndex] =  _GetBubbleImage(bMine, nIndex);
				}
			}

			if (pImage)
			{
				DrawNineBlockImage(graph,pImage,&rcBubble);
			}
		}
	}

	Image* CFTState:: _GetYourProgressbarImage(int nIndex)
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		return pResource->GetYourProgressBarImage(nIndex);
	}

	Image* CFTState::_GetMyProgressbarImage( int nIndex )
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		return pResource->GetMyProgressBarImage(nIndex);
	}

	Image* CFTState::_GetBubbleImage( BOOL bMine, int nIndex )
	{
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		if (bMine)
		{
			return pResource->GetMyBubbleImage(nIndex);
		}
		else
		{
			return pResource->GetOtherBubbleImage(nIndex);
		}
	}

	void CFTState::_DrawCancelButton( Graphics *pGraph, const PointF& ptGdiPOffset )
	{
		RectF rcDraw = m_rcButton[Button_General];
		rcDraw.Offset(ptGdiPOffset);
		IUIElementResouce* pResource = m_pUIChatFileElement->GetElementResource();
		if(CHAT_BUBBLE == pResource->GetChatDisplayType())
		{
			
			Image *cancel_btn = pResource->GetFileTransferImageByID(progress_cancel_gray);
			if (cancel_btn)
			{
				pGraph->DrawImage(cancel_btn, rcDraw);
			}
		}
		else if( CHAT_PLAIN == pResource->GetChatDisplayType())
		{
			CChatFileItem *pChatFileItem = dynamic_cast<CChatFileItem *>(m_pUIChatFileElement->GetChatFileItem());
			ASSERT(pChatFileItem);
			CString strCancel;
			strCancel.LoadString(IDS_STRING569);

			Gdiplus::FontStyle style =  FontStyleRegular;
			StringFormat format;
			format.SetAlignment(StringAlignmentNear);
			Gdiplus::Font txtfont(g_defaultFontFaceName, GetCurrentLayout(pResource->GetChatDisplayType()).common().font_height, style );
			SolidBrush txtbrush(Color::Blue);
			pGraph->DrawString(strCancel.GetString(), strCancel.GetLength(), &txtfont, rcDraw, &format,  &txtbrush);
		}
	}

	BOOL CFTState::_DrawProgressText( Graphics *pGraph, RectF& rcBar )
	{
		IUIElementResouce *pResource = m_pUIChatFileElement->GetElementResource();
		ASSERT(pResource);
		CChatFileItem *pChatFileItem = dynamic_cast<CChatFileItem*>(m_pUIChatFileElement->GetChatFileItem());
		ASSERT(pChatFileItem);

		int nProgress = pChatFileItem->GetTranslateFileInfo()->GetProgress();
		Gdiplus::FontStyle style = FontStyleBold;
		StringFormat format;
		format.SetAlignment(StringAlignmentNear);
		format.SetLineAlignment(StringAlignmentNear);
		Gdiplus::Font txtfont(DEFAULT_KOREAN_FONT_NAME_T, GetCurrentLayout(pResource->GetChatDisplayType()).common().font_height, style );
		SolidBrush txtbrush(GetCurrentLayout(pResource->GetChatDisplayType()).common().clr_progress_text);
		CString str;
		str.Format(_T("%d%%"), nProgress);
		pGraph->DrawString(str, str.GetLength(), &txtfont, rcBar, &format,  &txtbrush);
		return TRUE;
	}

	int CFTState::_GetBubbleCornerUnvisibleWidth()
	{
		return 5;
	}

	void CFTState::_CalcProgressSize( const CPoint& ptOffset )
	{
		int nChatType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();

		if(nChatType == CHAT_BUBBLE)
		{
			//_CalcProgressSize(ptOffset);
			//return;
			if (!m_strProgressInfo.IsEmpty())
			{
				/*int nInnerWidth = m_BubbleInnerWidth;
				CSize szBound = _CalcTextSize(m_strProgressInfo, ptOffset);
				m_rcProgress.X = (nInnerWidth / 2 + nLeftEdge - szBound.cx - 
					_GetBubbleCornerUnvisibleWidth()) / 2 + ptOffset.x;
				m_rcProgress.Width = (REAL)szBound.cx;
				m_rcProgress.Height = (REAL)szBound.cy;*/
				
			}
			else
			{
				int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
				m_rcProgress.X = (REAL)GetCurrentLayout(nDisplayType).common().progressbar_to_bubble_left;
				m_rcProgress.Y = (REAL)(m_rcFileIcon.GetBottom() - (_GetBubbleImage(FALSE, 5)->GetHeight() - _GetYourProgressbarImage(0)->GetWidth()) / 2 - _GetYourProgressbarImage(0)->GetWidth());
				m_rcProgress.Width = m_rcFileIcon.Width - m_rcProgress.X - (REAL)GetCurrentLayout(nDisplayType).common().progressbar_to_bubble_right;
				m_rcProgress.Height = (REAL)_GetYourProgressbarImage(0)->GetHeight();
			}
		}
		else
		{
			if (!m_strProgressInfo.IsEmpty())
			{
				CSize szBound = _CalcTextSize(m_strProgressInfo, ptOffset, GetCurrentLayout(nChatType).common().font_height);
				m_rcProgress.Width = (REAL)szBound.cx;
				m_rcProgress.Height = (REAL)szBound.cy;
			}
			else
			{
				int nDisplayType = m_pUIChatFileElement->GetElementResource()->GetChatDisplayType();
				m_rcProgress.Width = (REAL)GetCurrentLayout(nDisplayType).common().GetProgressbarWidth();
				m_rcProgress.Height = (REAL)GetCurrentLayout(nDisplayType).common().GetProgressbarHeight();
			}
		}
	}

	void CFTState::_CalcDividerSize( const CPoint& ptOffset )
	{

	}

	void CFTState::_CalcButtonsSize( const CPoint& ptOffset )
	{

	}

	BOOL CFTState::_DrawButton( Graphics *pGraph, const PointF& ptGdiPOffset )
	{
		return TRUE;
	}

	

}




